==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/entity/ThrowableCopyEntity.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.entity;

import net.minecraft.class_1299;
import net.minecraft.class_1309;
import net.minecraft.class_1675;
import net.minecraft.class_1676;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_239;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_2586;
import net.minecraft.class_2643;
import net.minecraft.class_2680;
import net.minecraft.class_3965;

// [VanillaCopy] ThrowableProjectile
public abstract class ThrowableCopyEntity extends class_1676 {
	protected ThrowableCopyEntity(class_1299<? extends ThrowableCopyEntity> entityType, class_1937 level) {
		super(entityType, level);
	}

	protected ThrowableCopyEntity(class_1299<? extends ThrowableCopyEntity> entityType, double d, double e, double f, class_1937 level) {
		this(entityType, level);
		this.method_5814(d, e, f);
	}

	protected ThrowableCopyEntity(class_1299<? extends ThrowableCopyEntity> entityType, class_1309 livingEntity, class_1937 level) {
		this(entityType, livingEntity.method_23317(), livingEntity.method_23320() - 0.10000000149011612D, livingEntity.method_23321(), level);
		this.method_7432(livingEntity);
	}

	@Override
	public boolean method_5640(double d) {
		double e = this.method_5829().method_995() * 4.0D;
		if (Double.isNaN(e)) {
			e = 4.0D;
		}

		e *= 64.0D;
		return d < e * e;
	}

	@Override
	public void method_5773() {
		super.method_5773();
		class_239 hitResult = class_1675.method_18074(this, this::method_26958);
		boolean bl = false;
		if (hitResult.method_17783() == class_239.class_240.field_1332) {
			class_2338 blockPos = ((class_3965) hitResult).method_17777();
			class_2680 blockState = this.field_6002.method_8320(blockPos);
			if (blockState.method_27852(class_2246.field_10316)) {
				this.method_5717(blockPos);
				bl = true;
			} else if (blockState.method_27852(class_2246.field_10613)) {
				class_2586 blockEntity = this.field_6002.method_8321(blockPos);
				if (blockEntity instanceof class_2643 gateway && class_2643.method_30276(this)) {
					class_2643.method_11409(this.field_6002, blockPos, blockState, this, gateway);
				}

				bl = true;
			}
		}

		if (hitResult.method_17783() != class_239.class_240.field_1333 && !bl) {
			this.method_7488(hitResult);
		}

		this.method_5852();
		class_243 vec3 = this.method_18798();
		double d = this.method_23317() + vec3.field_1352;
		double e = this.method_23318() + vec3.field_1351;
		double f = this.method_23321() + vec3.field_1350;
		this.method_26962();
		float j;
		if (this.method_5799()) {
			for (int i = 0; i < 4; ++i) {
				float g = 0.25F;
				this.field_6002.method_8406(class_2398.field_11247, d - vec3.field_1352 * 0.25D, e - vec3.field_1351 * 0.25D, f - vec3.field_1350 * 0.25D, vec3.field_1352, vec3.field_1351, vec3.field_1350);
			}

			j = 0.8F;
		} else {
			j = 0.99F;
		}

		this.method_18799(vec3.method_1021((double) j));
		if (!this.method_5740()) {
			class_243 vec32 = this.method_18798();
			this.method_18800(vec32.field_1352, vec32.field_1351 - (double) this.getGravity(), vec32.field_1350);
		}

		this.method_5814(d, e, f);
	}

	protected float getGravity() {
		return 0.03F;
	}
}

==> Botania-1.20.1-449-FABRIC-20250803.055636-52-sources.jar: vazkii/botania/common/entity/ThrowableCopyEntity.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.entity;

import net.minecraft.class_1299;
import net.minecraft.class_1309;
import net.minecraft.class_1675;
import net.minecraft.class_1676;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_239;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_2586;
import net.minecraft.class_2643;
import net.minecraft.class_2680;
import net.minecraft.class_3965;

// [VanillaCopy] ThrowableProjectile
public abstract class ThrowableCopyEntity extends class_1676 {
	protected ThrowableCopyEntity(class_1299<? extends ThrowableCopyEntity> entityType, class_1937 level) {
		super(entityType, level);
	}

	protected ThrowableCopyEntity(class_1299<? extends ThrowableCopyEntity> entityType, double d, double e, double f, class_1937 level) {
		this(entityType, level);
		this.method_5814(d, e, f);
	}

	protected ThrowableCopyEntity(class_1299<? extends ThrowableCopyEntity> entityType, class_1309 livingEntity, class_1937 level) {
		this(entityType, livingEntity.method_23317(), livingEntity.method_23320() - 0.10000000149011612D, livingEntity.method_23321(), level);
		this.method_7432(livingEntity);
	}

	@Override
	public boolean method_5640(double d) {
		double e = this.method_5829().method_995() * 4.0D;
		if (Double.isNaN(e)) {
			e = 4.0D;
		}

		e *= 64.0D;
		return d < e * e;
	}

	@Override
	public void method_5773() {
		super.method_5773();
		class_239 hitResult = class_1675.method_49997(this, this::method_26958);
		boolean bl = false;
		if (hitResult.method_17783() == class_239.class_240.field_1332) {
			class_2338 blockPos = ((class_3965) hitResult).method_17777();
			class_2680 blockState = this.method_37908().method_8320(blockPos);
			if (blockState.method_27852(class_2246.field_10316)) {
				this.method_5717(blockPos);
				bl = true;
			} else if (blockState.method_27852(class_2246.field_10613)) {
				class_2586 blockEntity = this.method_37908().method_8321(blockPos);
				if (blockEntity instanceof class_2643 gateway && class_2643.method_30276(this)) {
					class_2643.method_11409(this.method_37908(), blockPos, blockState, this, gateway);
				}

				bl = true;
			}
		}

		if (hitResult.method_17783() != class_239.class_240.field_1333 && !bl) {
			this.method_7488(hitResult);
		}

		this.method_5852();
		class_243 vec3 = this.method_18798();
		double d = this.method_23317() + vec3.field_1352;
		double e = this.method_23318() + vec3.field_1351;
		double f = this.method_23321() + vec3.field_1350;
		this.method_26962();
		float j;
		if (this.method_5799()) {
			for (int i = 0; i < 4; ++i) {
				float g = 0.25F;
				this.method_37908().method_8406(class_2398.field_11247, d - vec3.field_1352 * 0.25D, e - vec3.field_1351 * 0.25D, f - vec3.field_1350 * 0.25D, vec3.field_1352, vec3.field_1351, vec3.field_1350);
			}

			j = 0.8F;
		} else {
			j = 0.99F;
		}

		this.method_18799(vec3.method_1021((double) j));
		if (!this.method_5740()) {
			class_243 vec32 = this.method_18798();
			this.method_18800(vec32.field_1352, vec32.field_1351 - (double) this.getGravity(), vec32.field_1350);
		}

		this.method_5814(d, e, f);
	}

	protected float getGravity() {
		return 0.03F;
	}
}
